# detection type
BBOX = 'bbox'

# Feedback type
FEEDBACK_BBOX = 'feedback_bbox'
FEEDBACK_FALSE_ALERT = 'feedback_false_alert'


# image producer type
POST_PATH = 'post_path'
POST_BYTES = 'post_bytes'
IN_MEMORY = 'in_memory'
